/*
Copyright (C) Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <qcheckbox.h>

#include "ExportVHDLDlgImpl.h"
#include "Options.h"

/* 
 *  Constructs a ExportVHDLDlgImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
ExportVHDLDlgImpl::ExportVHDLDlgImpl( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : ExportVHDLDlg( parent, name, modal, fl )
{
}

/*  
 *  Destroys the object and frees any allocated resources
 */
ExportVHDLDlgImpl::~ExportVHDLDlgImpl()
{
    // no need to delete child widgets, Qt does it all for us
}

/*
 * public slot
 */
void ExportVHDLDlgImpl::symbolicNamesClicked()
{
  symbolic_states = cb_symbolicstates->isChecked();
}

/*
 * public slot
 */
void ExportVHDLDlgImpl::synchronousResetClicked()
{
  sync_reset = cb_synchreset->isChecked();
}

/*
 * public slot
 */
void ExportVHDLDlgImpl::useStdLogicClicked()
{
  stdlogic = cb_stdlogic->isChecked();
}


/**
 * Initialises the dialog according to the current options @a opt.
 */
void ExportVHDLDlgImpl::init(Options* opt)
{
  symbolic_states = opt->getVHDLSymbolicStates();
  cb_symbolicstates->setChecked(symbolic_states);
  sync_reset = opt->getVHDLSyncReset();
  cb_synchreset->setChecked(sync_reset);
  stdlogic = opt->getVHDLStdLogic();
  cb_stdlogic->setChecked(stdlogic);
}
