/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdlib.h>
#include <qdir.h>
#include <qstringlist.h>

#include "OptGeneralDlgImpl.h"

/**
 *  Constructs a OptGeneralDlgImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
OptGeneralDlgImpl::OptGeneralDlgImpl( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : OptGeneralDlg( parent, name, modal, fl )
{
}

/**  
 *  Destroys the object and frees any allocated resources
 */
OptGeneralDlgImpl::~OptGeneralDlgImpl()
{
}

/// Returns the language set in the combo box
QString OptGeneralDlgImpl::getLanguage()
{ 
  QString tmp;
  if (cmb_language->count()==0)
    return "English";

  tmp = cmb_language->currentText(); 
  return tmp;
}


/**
 * Sets the language @a s in the combo box.
 */
void OptGeneralDlgImpl::setLanguage(QString s)
{
  QString tmp;

  for (int i=0; i<cmb_language->count(); i++)
  {
    tmp = cmb_language->text(i);
    if (tmp==s)
      cmb_language->setCurrentItem(i);
  }
}

/**
 * Initialises the dialog with the current language
 */
void OptGeneralDlgImpl::init()
{
  QDir dir;
  QStringList flist, list;
  QString fn;
  int len;

  QString apppath=getenv("QFSMDIR");
  if (!apppath)
    qWarning("$QFSMDIR not set!");
  dir = QDir(apppath); //QDir::current();
  dir.cd("po");
  dir.setNameFilter("*.qm");
  flist = dir.entryList();

  cmb_language->clear();

  QStringList::Iterator it;

  for (it=flist.begin(); it!=flist.end(); ++it)
  {
    fn = *it;
    len = fn.length();
    fn = fn.left(len-3);
    list.append(fn);
  }

  if (list.count()>0)
    cmb_language->insertStringList(list);
}
